package com.me.brickbuster.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.EdgeShape;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.EntityType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;
import net.dermetfan.utils.Pair;

public class Paddle extends Entity implements PhysicsBody {

	public static final Color STICKY_COLOR = Color.GRAY;
	public static final Color PADDLE_COLOR = Color.WHITE;

	public static final float DEFAULT_WIDTH = 7.5f;
	public static final float PADDLE_HEIGHT = 0.75f;
	public static final float PADDLE_Y = 1.25f;
	public static final float PADDLE_SPEED = 40f;

	private float width = DEFAULT_WIDTH;
	private boolean sticky = false;

	private Body body;

	public Paddle(PlayState brickBuster) {
		super(brickBuster, PlayState.BOARD_WIDTH/2, PADDLE_Y);
		createBody();
	}

	@Override
	public void render(SpriteBatch sb, ShapeRenderer sr) {
		sr.begin(ShapeType.Filled);
		sr.setColor(sticky? STICKY_COLOR : PADDLE_COLOR);
		sr.rect(pos.x - width/2, pos.y, width, PADDLE_HEIGHT);
		sr.end();
	}

	@Override
	public void update(float dt) {
		float displacement = PADDLE_SPEED * dt;
		float adjust = 0;
		if (Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
			if (pos.x - width / 2 - displacement < PlayState.EDGE_PADDING) {
				adjust = -(pos.x - width/2 - PlayState.EDGE_PADDING);
			} else {
				adjust = -displacement;
			}
		}
		if (Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
			if (pos.x + width / 2 + displacement > PlayState.BOARD_WIDTH-PlayState.EDGE_PADDING) {
				adjust = PlayState.BOARD_WIDTH - PlayState.EDGE_PADDING - width/2 - pos.x;
			} else {
				adjust = displacement;
			}
		}
		if (!MathUtils.isZero(adjust)) {
			setX(pos.x + adjust);
			for (Ball ball : ((PlayState) state).balls) {
				if (ball.isStuck()) {
					ball.setX(ball.getX() + adjust);
				}
			}
		}
	}

	@Override
	public void createBody() {
		BodyDef paddleBody = new BodyDef();
		paddleBody.type = BodyDef.BodyType.KinematicBody;
		paddleBody.position.set(pos);

		EdgeShape paddleShape = new EdgeShape();
		paddleShape.set(new Vector2(-width/2, PADDLE_HEIGHT),
				new Vector2(width/2, PADDLE_HEIGHT));

		FixtureDef paddleFixture = new FixtureDef();
		paddleFixture.shape = paddleShape;

		paddleFixture.filter.categoryBits = EntityType.PADDLE;
		paddleFixture.filter.maskBits = EntityType.BALL | EntityType.POWER_UP;

		body = state.world.createBody(paddleBody);
		body.createFixture(paddleFixture);
		body.setUserData(this);

		paddleShape.dispose();
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void setX(float x) {
		super.setX(x);
		body.setTransform(pos, 0);
	}

	@Override
	public void setY(float y) {
		super.setY(y);
		body.setTransform(pos, 0);
	}

	public float getWidth() {
		return width;
	}

	public void setWidth(float width) {
		if (this.width == width) {
			return;
		}
		((PlayState) state).world.destroyBody(body);
		this.width = width;
		createBody();
	}

	public boolean isSticky() {
		return sticky;
	}

	public void setSticky(boolean sticky) {
		this.sticky = sticky;
	}




}
